# Generated by Django 4.0.5 on 2022-07-03 13:04

from django.db import migrations
import edc_model.models.fields.other_charfield


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0038_alter_endofstudy_delivery_date_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicaloffstudymedication',
            name='reason_other',
            field=edc_model.models.fields.other_charfield.OtherCharField(blank=True, max_length=35, null=True, verbose_name='If other, please specify ...'),
        ),
        migrations.AddField(
            model_name='offstudymedication',
            name='reason_other',
            field=edc_model.models.fields.other_charfield.OtherCharField(blank=True, max_length=35, null=True, verbose_name='If other, please specify ...'),
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-29 15:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0032_historicalegfrnotification_egfrnotification'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalegfrnotification',
            name='action_item',
        ),
        migrations.RemoveField(
            model_name='historicalegfrnotification',
            name='history_user',
        ),
        migrations.RemoveField(
            model_name='historicalegfrnotification',
            name='parent_action_item',
        ),
        migrations.RemoveField(
            model_name='historicalegfrnotification',
            name='related_action_item',
        ),
        migrations.RemoveField(
            model_name='historicalegfrnotification',
            name='site',
        ),
        migrations.DeleteModel(
            name='EgfrNotification',
        ),
        migrations.DeleteModel(
            name='HistoricalEgfrNotification',
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-27 14:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0030_auto_20220627_1119'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicaloffschedule',
            options={'get_latest_by': 'history_date', 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Off-schedule'},
        ),
        migrations.AlterModelOptions(
            name='historicaloffschedulepostnatal',
            options={'get_latest_by': 'history_date', 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Off-schedule: post-natal'},
        ),
        migrations.AlterModelOptions(
            name='historicaloffschedulepregnancy',
            options={'get_latest_by': 'history_date', 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Off-schedule: Pregnancy'},
        ),
        migrations.AlterModelOptions(
            name='offschedule',
            options={'verbose_name': 'Off-schedule', 'verbose_name_plural': 'Off-schedule'},
        ),
        migrations.AlterModelOptions(
            name='offschedulepostnatal',
            options={'verbose_name': 'Off-schedule: post-natal', 'verbose_name_plural': 'Off-schedule: post-natal'},
        ),
        migrations.AlterModelOptions(
            name='offschedulepregnancy',
            options={'verbose_name': 'Off-schedule: Pregnancy', 'verbose_name_plural': 'Off-schedule: Pregnancy'},
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-30 11:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0035_auto_20220630_1140'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='endofstudy',
            name='meta_prn_en_id_a50384_idx',
        ),
        migrations.RenameField(
            model_name='endofstudy',
            old_name='offschedule_datetime',
            new_name='offstudy_datetime',
        ),
        migrations.RenameField(
            model_name='endofstudy',
            old_name='offschedule_reason',
            new_name='offstudy_reason',
        ),
        migrations.RenameField(
            model_name='endofstudy',
            old_name='other_offschedule_reason',
            new_name='other_offstudy_reason',
        ),
        migrations.RenameField(
            model_name='historicalendofstudy',
            old_name='offschedule_datetime',
            new_name='offstudy_datetime',
        ),
        migrations.RenameField(
            model_name='historicalendofstudy',
            old_name='offschedule_reason',
            new_name='offstudy_reason',
        ),
        migrations.RenameField(
            model_name='historicalendofstudy',
            old_name='other_offschedule_reason',
            new_name='other_offstudy_reason',
        ),
    ]



from django.contrib import admin
from edc_action_item import ActionItemModelAdminMixin, action_fieldset_tuple
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import meta_prn_admin
from ..forms import OffstudyMedicationForm
from ..models import OffstudyMedication


@admin.register(OffstudyMedication, site=meta_prn_admin)
class OffstudyMedicationAdmin(
    DataManagerModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    ActionItemModelAdminMixin,
    SimpleHistoryAdmin,
):

    form = OffstudyMedicationForm

    fieldsets = (
        (None, {"fields": ("subject_identifier", "report_datetime")}),
        (
            "Loss to followup",
            {
                "fields": (
                    "stop_date",
                    "last_dose_date",
                    "reason",
                    "reason_other",
                    "comment",
                )
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_identifier",
        "dashboard",
        "stop_date",
        "last_dose_date",
    )

    list_filter = (
        "reason",
        "stop_date",
        "last_dose_date",
    )

    radio_fields = {
        "reason": admin.VERTICAL,
    }

    search_fields = ("subject_identifier", "action_identifier", "tracking_identifier")



from django.contrib import admin, messages
from edc_lab_results.admin import BloodResultsModelAdminMixin
from edc_lab_results.fieldsets import BloodResultFieldset

from ...admin_site import meta_subject_admin
from ...forms import BloodResultsRftForm
from ...models import BloodResultsRft, EgfrNotification
from ..modeladmin import CrfModelAdmin


@admin.register(BloodResultsRft, site=meta_subject_admin)
class BloodResultsRftAdmin(BloodResultsModelAdminMixin, CrfModelAdmin):

    form = BloodResultsRftForm
    actions = ["create_or_update_egfr_notification"]
    fieldsets = (
        *BloodResultFieldset(
            BloodResultsRft.lab_panel,
            model_cls=BloodResultsRft,
            excluded_utest_ids=["egfr"],
        ).fieldsets,
        (
            "Calculated eGFR",
            {
                "classes": ("collapse",),
                "fields": ["egfr_value", "egfr_units", "egfr_percent_change"],
            },
        ),
    )

    readonly_fields = ["egfr_value", "egfr_units", "egfr_percent_change"]

    @admin.action(permissions=["view"], description="Create or update eGFR notification")
    def create_or_update_egfr_notification(self, request, queryset):
        total = EgfrNotification.objects.all().count()
        for obj in queryset:
            obj.save()
        new_total = EgfrNotification.objects.all().count()
        messages.success(request, f"Created {new_total - total} new eGFR notifications.")



# Generated by Django 4.0.5 on 2022-06-29 15:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_subject', '0115_historicalegfrnotification_egfrnotification'),
    ]

    operations = [
        migrations.AddField(
            model_name='egfrnotification',
            name='report_status',
            field=models.CharField(choices=[('New', 'New'), ('open', 'Open. Some information is still pending.'), ('closed', 'Closed. This report is complete')], default='New', max_length=15),
        ),
        migrations.AddField(
            model_name='historicalegfrnotification',
            name='report_status',
            field=models.CharField(choices=[('New', 'New'), ('open', 'Open. Some information is still pending.'), ('closed', 'Closed. This report is complete')], default='New', max_length=15),
        ),
    ]



# Generated by Django 4.0.5 on 2022-07-03 08:41

from django.db import migrations
import edc_sites.models
import meta_subject.models.birth_outcomes


class Migration(migrations.Migration):

    dependencies = [
        ('meta_subject', '0119_historicalstudymedication_roundup_divisible_by_and_more'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='birthoutcomes',
            managers=[
                ('on_site', edc_sites.models.CurrentSiteManager()),
                ('objects', meta_subject.models.birth_outcomes.Manager()),
            ],
        ),
        migrations.RemoveField(
            model_name='birthoutcomes',
            name='action_identifier',
        ),
        migrations.RemoveField(
            model_name='birthoutcomes',
            name='action_item',
        ),
        migrations.RemoveField(
            model_name='birthoutcomes',
            name='action_item_reason',
        ),
        migrations.RemoveField(
            model_name='birthoutcomes',
            name='parent_action_identifier',
        ),
        migrations.RemoveField(
            model_name='birthoutcomes',
            name='parent_action_item',
        ),
        migrations.RemoveField(
            model_name='birthoutcomes',
            name='related_action_identifier',
        ),
        migrations.RemoveField(
            model_name='birthoutcomes',
            name='related_action_item',
        ),
        migrations.RemoveField(
            model_name='birthoutcomes',
            name='subject_visit',
        ),
        migrations.RemoveField(
            model_name='birthoutcomes',
            name='tracking_identifier',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='action_identifier',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='action_item',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='action_item_reason',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='parent_action_identifier',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='parent_action_item',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='related_action_identifier',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='related_action_item',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='subject_visit',
        ),
        migrations.RemoveField(
            model_name='historicalbirthoutcomes',
            name='tracking_identifier',
        ),
    ]



from copy import copy

from django.contrib import admin
from django.utils.html import format_html
from edc_action_item import action_fields, action_fieldset_tuple
from edc_constants.constants import CLOSED, OPEN
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import meta_prn_admin
from ..forms import ProtocolIncidentForm
from ..models import ProtocolIncident


@admin.register(ProtocolIncident, site=meta_prn_admin)
class ProtocolDeviationViolationAdmin(
    DataManagerModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):

    form = ProtocolIncidentForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "report_datetime",
                    "short_description",
                    "report_type",
                )
            },
        ),
        (
            "Details of incident",
            {
                "fields": (
                    "safety_impact",
                    "safety_impact_details",
                    "study_outcomes_impact",
                    "study_outcomes_impact_details",
                    "incident_datetime",
                    "incident",
                    "incident_other",
                    "incident_description",
                    "incident_reason",
                ),
            },
        ),
        (
            "Actions taken",
            {
                "description": (
                    "The following questions are required before the report is closed."
                ),
                "fields": (
                    "corrective_action_datetime",
                    "corrective_action",
                    "preventative_action_datetime",
                    "preventative_action",
                    "action_required",
                ),
            },
        ),
        ("Report status", {"fields": ("report_status", "report_closed_datetime")}),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "action_required": admin.VERTICAL,
        "report_status": admin.VERTICAL,
        "report_type": admin.VERTICAL,
        "safety_impact": admin.VERTICAL,
        "study_outcomes_impact": admin.VERTICAL,
        "incident": admin.VERTICAL,
    }

    list_display = (
        "subject_identifier",
        "dashboard",
        "description",
        "report_datetime",
        "status",
        "action_required",
        "report_type",
        "tracking_identifier",
        "action_identifier",
        "user_created",
    )

    list_filter = ("action_required", "report_status", "report_type")

    search_fields = [
        "subject_identifier",
        "action_identifier",
        "short_description",
        "tracking_identifier",
    ]

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        action_flds.remove("action_identifier")
        fields = list(action_flds) + list(fields)
        return fields

    def status(self, obj=None):
        if obj.report_status == CLOSED:
            return format_html(f'<font color="green">{obj.report_status.title()}</font>')
        elif obj.report_status == OPEN:
            return format_html(f'<font color="red">{obj.report_status.title()}</font>')
        return obj.report_status.title()

    def description(self, obj=None):
        return obj.short_description.title()



# Generated by Django 4.0.5 on 2022-07-03 19:15

import _socket
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_crypto_fields.fields.encrypted_text_field
import django_revision.revision_field
import edc_action_item.models.action_model_mixin
import edc_model.models.fields.other_charfield
import edc_model.validators.date
import edc_sites.models
import edc_utils.date
import simple_history.models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('edc_action_item', '0028_auto_20210203_0706'),
        ('meta_lists', '0013_transferreasons_and_more'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('meta_prn', '0040_remove_historicaloffstudymedication_expected_restart_date_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='endofstudy',
            name='transfer_date',
            field=models.DateField(blank=True, help_text='A Transfer form must be on file.', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date of transfer, if applicable'),
        ),
        migrations.AddField(
            model_name='historicalendofstudy',
            name='transfer_date',
            field=models.DateField(blank=True, help_text='A Transfer form must be on file.', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date of transfer, if applicable'),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='clinical_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('kidney_disease', 'Development of chronic kidney disease'), ('liver_disease', 'Development of chronic liver disease'), ('intercurrent_illness', 'Intercurrent illness which prevents further treatment'), ('investigator_decision', 'Investigator decision (specify below)'), ('OTHER', 'Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)')], max_length=25, null=True, verbose_name='If withdrawn on CLINICAL grounds, please specify PRIMARY reason...'),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='offstudy_reason',
            field=models.ForeignKey(limit_choices_to={'name__in': ['clinical_withdrawal', 'completed_followup', 'dead', 'delivery', 'diabetes', 'late_exclusion', 'LTFU', 'OTHER', 'pregnancy', 'toxicity', 'transferred', 'withdrawal']}, null=True, on_delete=django.db.models.deletion.PROTECT, to='meta_lists.offstudyreasons', verbose_name='Reason patient was terminated from the study'),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='toxicity_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('lactic_acidosis', 'Development of lactic acidosis or hyperlactatemia'), ('hepatomegaly', 'Development of hepatomegaly with steatosis'), ('OTHER', 'Other (specify below)')], max_length=25, null=True, verbose_name=' Patient experienced an unacceptable toxicity, please specify ...'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='clinical_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('kidney_disease', 'Development of chronic kidney disease'), ('liver_disease', 'Development of chronic liver disease'), ('intercurrent_illness', 'Intercurrent illness which prevents further treatment'), ('investigator_decision', 'Investigator decision (specify below)'), ('OTHER', 'Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)')], max_length=25, null=True, verbose_name='If withdrawn on CLINICAL grounds, please specify PRIMARY reason...'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='offstudy_reason',
            field=models.ForeignKey(blank=True, db_constraint=False, limit_choices_to={'name__in': ['clinical_withdrawal', 'completed_followup', 'dead', 'delivery', 'diabetes', 'late_exclusion', 'LTFU', 'OTHER', 'pregnancy', 'toxicity', 'transferred', 'withdrawal']}, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='meta_lists.offstudyreasons', verbose_name='Reason patient was terminated from the study'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='toxicity_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('lactic_acidosis', 'Development of lactic acidosis or hyperlactatemia'), ('hepatomegaly', 'Development of hepatomegaly with steatosis'), ('OTHER', 'Other (specify below)')], max_length=25, null=True, verbose_name=' Patient experienced an unacceptable toxicity, please specify ...'),
        ),
        migrations.CreateModel(
            name='SubjectTransfer',
            fields=[
                ('revision', django_revision.revision_field.RevisionField(blank=True, editable=False, help_text='System field. Git repository tag:branch:commit.', max_length=75, null=True, verbose_name='Revision')),
                ('created', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('modified', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('user_created', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user created')),
                ('user_modified', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user modified')),
                ('hostname_created', models.CharField(blank=True, default=_socket.gethostname, help_text='System field. (modified on create only)', max_length=60)),
                ('hostname_modified', django_audit_fields.fields.hostname_modification_field.HostnameModificationField(blank=True, help_text='System field. (modified on every save)', max_length=50)),
                ('device_created', models.CharField(blank=True, max_length=10)),
                ('device_modified', models.CharField(blank=True, max_length=10)),
                ('id', django_audit_fields.fields.uuid_auto_field.UUIDAutoField(blank=True, editable=False, help_text='System auto field. UUID primary key.', primary_key=True, serialize=False)),
                ('subject_identifier', models.CharField(max_length=50, unique=True)),
                ('tracking_identifier', models.CharField(max_length=32, unique=True)),
                ('action_identifier', models.CharField(max_length=50, unique=True)),
                ('parent_action_identifier', models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True)),
                ('related_action_identifier', models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True)),
                ('action_item_reason', models.TextField(editable=False, null=True)),
                ('report_datetime', models.DateTimeField(default=edc_utils.date.get_utcnow, verbose_name='Report Date and Time')),
                ('transfer_date', models.DateField(default=edc_utils.date.get_utcnow, verbose_name='Transfer date')),
                ('initiated_by', models.CharField(choices=[('patient', 'Patient'), ('clinician', 'Attending Clinician / Healthcare worker'), ('OTHER', 'Other')], max_length=25, verbose_name='Who initiated the transfer request')),
                ('initiated_by_other', edc_model.models.fields.other_charfield.OtherCharField(blank=True, max_length=35, null=True, verbose_name='If other, please specify ...')),
                ('transfer_reason_other', edc_model.models.fields.other_charfield.OtherCharField(blank=True, max_length=35, null=True, verbose_name='If other, please specify ...')),
                ('may_return', models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('not_sure', 'Not sure')], max_length=15, verbose_name='Is the participant likely to transfer back before the end of their stay in the trial?')),
                ('may_contact', models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], max_length=15, verbose_name='Is the participant willing to be contacted at the end of the study?')),
                ('comment', django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(blank=True, help_text=' (Encryption: AES local)', max_length=71, verbose_name='Additional Comments')),
                ('action_item', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='edc_action_item.actionitem')),
                ('parent_action_item', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem')),
                ('related_action_item', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem')),
                ('site', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='sites.site')),
                ('transfer_reason', models.ManyToManyField(to='meta_lists.transferreasons', verbose_name='Reason for transfer')),
            ],
            options={
                'verbose_name': 'Subject Transfer',
                'verbose_name_plural': 'Subject Transfers',
                'ordering': ('-modified', '-created'),
                'get_latest_by': 'modified',
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'export', 'import'),
            },
            managers=[
                ('on_site', edc_sites.models.CurrentSiteManager()),
                ('objects', edc_action_item.models.action_model_mixin.ActionItemModelManager()),
            ],
        ),
        migrations.CreateModel(
            name='HistoricalSubjectTransfer',
            fields=[
                ('revision', django_revision.revision_field.RevisionField(blank=True, editable=False, help_text='System field. Git repository tag:branch:commit.', max_length=75, null=True, verbose_name='Revision')),
                ('created', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('modified', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('user_created', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user created')),
                ('user_modified', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user modified')),
                ('hostname_created', models.CharField(blank=True, default=_socket.gethostname, help_text='System field. (modified on create only)', max_length=60)),
                ('hostname_modified', django_audit_fields.fields.hostname_modification_field.HostnameModificationField(blank=True, help_text='System field. (modified on every save)', max_length=50)),
                ('device_created', models.CharField(blank=True, max_length=10)),
                ('device_modified', models.CharField(blank=True, max_length=10)),
                ('id', django_audit_fields.fields.uuid_auto_field.UUIDAutoField(blank=True, db_index=True, editable=False, help_text='System auto field. UUID primary key.')),
                ('subject_identifier', models.CharField(db_index=True, max_length=50)),
                ('tracking_identifier', models.CharField(db_index=True, max_length=32)),
                ('action_identifier', models.CharField(db_index=True, max_length=50)),
                ('parent_action_identifier', models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True)),
                ('related_action_identifier', models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True)),
                ('action_item_reason', models.TextField(editable=False, null=True)),
                ('history_id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('report_datetime', models.DateTimeField(default=edc_utils.date.get_utcnow, verbose_name='Report Date and Time')),
                ('transfer_date', models.DateField(default=edc_utils.date.get_utcnow, verbose_name='Transfer date')),
                ('initiated_by', models.CharField(choices=[('patient', 'Patient'), ('clinician', 'Attending Clinician / Healthcare worker'), ('OTHER', 'Other')], max_length=25, verbose_name='Who initiated the transfer request')),
                ('initiated_by_other', edc_model.models.fields.other_charfield.OtherCharField(blank=True, max_length=35, null=True, verbose_name='If other, please specify ...')),
                ('transfer_reason_other', edc_model.models.fields.other_charfield.OtherCharField(blank=True, max_length=35, null=True, verbose_name='If other, please specify ...')),
                ('may_return', models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('not_sure', 'Not sure')], max_length=15, verbose_name='Is the participant likely to transfer back before the end of their stay in the trial?')),
                ('may_contact', models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], max_length=15, verbose_name='Is the participant willing to be contacted at the end of the study?')),
                ('comment', django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(blank=True, help_text=' (Encryption: AES local)', max_length=71, verbose_name='Additional Comments')),
                ('history_date', models.DateTimeField()),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('action_item', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='edc_action_item.actionitem')),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('parent_action_item', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='edc_action_item.actionitem')),
                ('related_action_item', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='edc_action_item.actionitem')),
                ('site', models.ForeignKey(blank=True, db_constraint=False, editable=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='sites.site')),
            ],
            options={
                'verbose_name': 'historical Subject Transfer',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-26 12:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0027_rename_historicalprotocoldeviationviolation_historicalprotocolincident_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalpregnancynotification',
            name='bhcg_date',
            field=models.DateField(blank=True, help_text='Will be validated against a UPT result in the EDC.', null=True, verbose_name='βHCG result date'),
        ),
        migrations.AddField(
            model_name='pregnancynotification',
            name='bhcg_date',
            field=models.DateField(blank=True, help_text='Will be validated against a UPT result in the EDC.', null=True, verbose_name='βHCG result date'),
        ),
        migrations.AlterField(
            model_name='historicalpregnancynotification',
            name='bhcg_confirmed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], help_text='If YES, the UPT result must already be entered in the EDC.', max_length=5, verbose_name='Has the pregnancy been confirmed by urine βHCG?'),
        ),
        migrations.AlterField(
            model_name='pregnancynotification',
            name='bhcg_confirmed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], help_text='If YES, the UPT result must already be entered in the EDC.', max_length=5, verbose_name='Has the pregnancy been confirmed by urine βHCG?'),
        ),
    ]



from copy import copy

from django.contrib import admin
from edc_action_item import (
    ActionItemModelAdminMixin,
    action_fields,
    action_fieldset_tuple,
)
from edc_crf.fieldset import crf_status_fieldset
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_form_label import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple

from ..admin_site import meta_subject_admin
from ..forms import EgfrNotificationForm
from ..models import EgfrNotification
from .modeladmin import CrfModelAdminMixin


@admin.register(EgfrNotification, site=meta_subject_admin)
class EgfrNotificationAdmin(
    DataManagerModelAdminMixin,
    CrfModelAdminMixin,
    FormLabelModelAdminMixin,
    ActionItemModelAdminMixin,
    SimpleHistoryAdmin,
):

    form = EgfrNotificationForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "eGFR",
            {
                "fields": (
                    "creatinine_date",
                    "egfr_percent_change",
                )
            },
        ),
        (
            "Narrative",
            {"fields": ("narrative", "report_status")},
        ),
        crf_status_fieldset,
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = (
        "report_datetime",
        "report_status",
        "creatinine_date",
        "egfr_percent_change",
    )

    list_filter = ("site", "report_status", "report_datetime", "creatinine_date")

    radio_fields = {"report_status": admin.VERTICAL}

    search_fields = (
        "subject_visit__subject_identifier",
        "action_identifier",
        "tracking_identifier",
    )

    readonly_fields = [
        "creatinine_date",
        "egfr_percent_change",
    ]

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        action_flds.remove("action_identifier")
        fields = list(action_flds) + list(fields)
        return fields



# Generated by Django 4.0.5 on 2022-06-27 18:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0047_historicalscreeningpartone_appt_datetime_repeat_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='historicalscreeningpartone',
            old_name='appt_datetime_repeat',
            new_name='repeat_appt_datetime',
        ),
        migrations.RenameField(
            model_name='historicalscreeningpartthree',
            old_name='appt_datetime_repeat',
            new_name='repeat_appt_datetime',
        ),
        migrations.RenameField(
            model_name='historicalscreeningparttwo',
            old_name='appt_datetime_repeat',
            new_name='repeat_appt_datetime',
        ),
        migrations.RenameField(
            model_name='historicalsubjectscreening',
            old_name='appt_datetime_repeat',
            new_name='repeat_appt_datetime',
        ),
        migrations.RenameField(
            model_name='subjectscreening',
            old_name='appt_datetime_repeat',
            new_name='repeat_appt_datetime',
        ),
    ]



from django.db import models
from edc_model import models as edc_models
from edc_pharmacy.models import StudyMedicationCrfModelMixin

from ..model_mixins import CrfModelMixin


class StudyMedication(StudyMedicationCrfModelMixin, CrfModelMixin, edc_models.BaseUuidModel):

    roundup_divisible_by = models.IntegerField(default=32)

    # first_dose_date = models.DateField(
    #     verbose_name="Date of first dose",
    #     validators=[date_not_before_study_start, date_not_future],
    #     default=get_utcnow,
    # )
    #
    # sid = models.CharField(
    #     verbose_name="SID# on pill bottles",
    #     max_length=15,
    #     validators=[RegexValidator(regex=r"^\w+$")],
    #     null=True,
    #     blank=False,
    # )

    class Meta(
        StudyMedicationCrfModelMixin.Meta,
        CrfModelMixin.Meta,
        edc_models.BaseUuidModel.Meta,
    ):
        pass



from copy import copy

from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_action_item import action_fields, action_fieldset_tuple
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import meta_prn_admin
from ..models import OffSchedulePostnatal


@admin.register(OffSchedulePostnatal, site=meta_prn_admin)
class OffSchedulePostnatalAdmin(
    DataManagerModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):

    instructions = None

    fieldsets = (
        (None, {"fields": ("subject_identifier", "offschedule_datetime")}),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = ("subject_identifier", "dashboard", "offschedule_datetime")

    list_filter = ("offschedule_datetime",)

    readonly_fields = ["subject_identifier", "offschedule_datetime"]

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        fields = list(action_flds) + list(fields)
        return fields



# Generated by Django 3.2.4 on 2021-06-23 23:25

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_action_item", "0028_auto_20210203_0706"),
        ("meta_consent", "0003_auto_20200325_0901"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalsubjectconsent",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Subject Consent",
            },
        ),
        migrations.AlterModelOptions(
            name="subjectconsent",
            options={
                "get_latest_by": "consent_datetime",
                "ordering": ("created",),
                "verbose_name": "Subject Consent",
                "verbose_name_plural": "Subject Consents",
            },
        ),
        migrations.AlterField(
            model_name="historicalsubjectreconsent",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectreconsent",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="subjectreconsent",
            name="action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="subjectreconsent",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="subjectreconsent",
            name="parent_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="subjectreconsent",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="subjectreconsent",
            name="related_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
    ]



from django import forms
from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_form_validators.form_validator import FormValidator
from edc_form_validators.form_validator_mixin import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin
from edc_visit_schedule.modelform_mixins import OffScheduleModelFormMixin

from ..models import OffSchedulePregnancy


class OffSchedulePregnancyFormValidator(FormValidator):
    pass


class OffSchedulePregnancyForm(
    OffScheduleModelFormMixin,
    SiteModelFormMixin,
    FormValidatorMixin,
    ActionItemFormMixin,
    forms.ModelForm,
):

    form_validator_cls = OffSchedulePregnancyFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = OffSchedulePregnancy
        fields = "__all__"



from django import forms
from edc_form_validators.form_validator import FormValidator
from edc_form_validators.form_validator_mixin import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin

from ..models import OffstudyMedication


class OffScheduleFormValidator(FormValidator):
    pass


class OffstudyMedicationForm(
    SiteModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):

    form_validator_cls = OffScheduleFormValidator

    class Meta:
        model = OffstudyMedication
        fields = "__all__"
        help_text = {"subject_identifier": "(read-only)", "action_identifier": "(read-only)"}
        widgets = {
            "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from dateutil.relativedelta import relativedelta
from django.contrib import admin
from edc_action_item import action_fieldset_tuple
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from meta_consent.models import SubjectConsent

from ..admin_site import meta_prn_admin
from ..forms import EndOfStudyForm
from ..models import EndOfStudy


@admin.register(EndOfStudy, site=meta_prn_admin)
class EndOfStudyAdmin(
    DataManagerModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):

    form = EndOfStudyForm

    date_hierarchy = "offstudy_datetime"

    fieldsets = (
        [
            None,
            {
                "fields": (
                    "subject_identifier",
                    "offstudy_datetime",
                    "last_seen_date",
                )
            },
        ],
        [
            "Off-study reason",
            {
                "fields": (
                    "offstudy_reason",
                    "other_offstudy_reason",
                )
            },
        ],
        [
            "Dates from supporting documents (if applicable)",
            {
                "fields": (
                    "death_date",
                    "ltfu_date",
                    "transfer_date",
                    "pregnancy_date",
                    "delivery_date",
                )
            },
        ],
        [
            "Toxicity (if applicable)",
            {
                "description": (
                    "This section is applicable if the patient experienced an "
                    "unacceptable toxicity, as indicated above"
                ),
                "fields": (
                    "toxicity_withdrawal_reason",
                    "toxicity_withdrawal_reason_other",
                ),
            },
        ],
        [
            "Withdrawn on CLINICAL grounds (if applicable)",
            {
                "description": (
                    "This section is applicable if the patient was withdrawn "
                    "on clinical grounds, as indicated above"
                ),
                "fields": (
                    "clinical_withdrawal_reason",
                    "clinical_withdrawal_investigator_decision",
                    "clinical_withdrawal_reason_other",
                ),
            },
        ],
        [
            "Comment",
            {"fields": ("comment",)},
        ],
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "offstudy_reason": admin.VERTICAL,
        "clinical_withdrawal_reason": admin.VERTICAL,
        "toxicity_withdrawal_reason": admin.VERTICAL,
    }

    search_fields = ["subject_identifier"]

    list_filter = ["offstudy_reason", "last_seen_date"]

    list_display = [
        "subject_identifier",
        "terminated",
        "last_seen",
        "months",
        "reason",
    ]

    @admin.display(description="terminated", ordering="offstudy_datetime")
    def terminated(self, obj):
        return obj.offstudy_datetime.date()

    @admin.display(description="last_seen", ordering="last_seen_date")
    def last_seen(self, obj):
        return obj.last_seen_date

    @admin.display(description="reason", ordering="offstudy_reason")
    def reason(self, obj):
        return obj.offstudy_reason

    @admin.display(description="months")
    def months(self, obj):
        subject_consent = SubjectConsent.objects.get(subject_identifier=obj.subject_identifier)
        return relativedelta(obj.offstudy_datetime, subject_consent.consent_datetime).months



# Generated by Django 3.2.13 on 2022-07-04 15:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0052_alter_historicalscreeningpartone_p3_ltfu_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicalicpreferral',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical icp referral', 'verbose_name_plural': 'historical icp referrals'},
        ),
        migrations.AlterModelOptions(
            name='historicalscreeningpartone',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Subject Screening: Part One', 'verbose_name_plural': 'historical Subject Screening: Part One'},
        ),
        migrations.AlterModelOptions(
            name='historicalscreeningpartthree',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Subject Screening: Part Three', 'verbose_name_plural': 'historical Subject Screening: Part Three'},
        ),
        migrations.AlterModelOptions(
            name='historicalscreeningparttwo',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Subject Screening: Part Two', 'verbose_name_plural': 'historical Subject Screening: Part Two'},
        ),
        migrations.AlterModelOptions(
            name='historicalsubjectrefusal',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Subject Refusal', 'verbose_name_plural': 'historical Subject Refusals'},
        ),
        migrations.AlterModelOptions(
            name='historicalsubjectscreening',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Subject Screening', 'verbose_name_plural': 'historical Subject Screening'},
        ),
        migrations.AlterField(
            model_name='historicalicpreferral',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalsubjectrefusal',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-26 13:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0028_historicalpregnancynotification_bhcg_date_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalpregnancynotification',
            name='edd',
            field=models.DateField(verbose_name='Estimated date of delivery'),
        ),
        migrations.AlterField(
            model_name='pregnancynotification',
            name='edd',
            field=models.DateField(verbose_name='Estimated date of delivery'),
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-28 10:24

import django.core.validators
from django.db import migrations, models
import django_crypto_fields.fields.encrypted_char_field


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0048_rename_appt_datetime_repeat_historicalscreeningpartone_repeat_appt_datetime_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='p3_ltfu',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='Only applicable if the subject missed the appointment for the second stage of screening', max_length=15, verbose_name='Should additional attempts be made to contact this subject'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='p3_ltfu_comment',
            field=models.TextField(null=True, verbose_name='Provide any additional comments on this decision (or leave blank)'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='p3_ltfu_date',
            field=models.DateField(blank=True, help_text='Must be after the appointment date for the second stage of screening', null=True, verbose_name='Date decision made'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='p3_ltfu',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='Only applicable if the subject missed the appointment for the second stage of screening', max_length=15, verbose_name='Should additional attempts be made to contact this subject'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='p3_ltfu_comment',
            field=models.TextField(null=True, verbose_name='Provide any additional comments on this decision (or leave blank)'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='p3_ltfu_date',
            field=models.DateField(blank=True, help_text='Must be after the appointment date for the second stage of screening', null=True, verbose_name='Date decision made'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='p3_ltfu',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='Only applicable if the subject missed the appointment for the second stage of screening', max_length=15, verbose_name='Should additional attempts be made to contact this subject'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='p3_ltfu_comment',
            field=models.TextField(null=True, verbose_name='Provide any additional comments on this decision (or leave blank)'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='p3_ltfu_date',
            field=models.DateField(blank=True, help_text='Must be after the appointment date for the second stage of screening', null=True, verbose_name='Date decision made'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='p3_ltfu',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='Only applicable if the subject missed the appointment for the second stage of screening', max_length=15, verbose_name='Should additional attempts be made to contact this subject'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='p3_ltfu_comment',
            field=models.TextField(null=True, verbose_name='Provide any additional comments on this decision (or leave blank)'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='p3_ltfu_date',
            field=models.DateField(blank=True, help_text='Must be after the appointment date for the second stage of screening', null=True, verbose_name='Date decision made'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='p3_ltfu',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='Only applicable if the subject missed the appointment for the second stage of screening', max_length=15, verbose_name='Should additional attempts be made to contact this subject'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='p3_ltfu_comment',
            field=models.TextField(null=True, verbose_name='Provide any additional comments on this decision (or leave blank)'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='p3_ltfu_date',
            field=models.DateField(blank=True, help_text='Must be after the appointment date for the second stage of screening', null=True, verbose_name='Date decision made'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='contact_number',
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(blank=True, help_text='Provide a contact number if repeating glucose measures (Encryption: RSA local)', max_length=71, null=True, validators=[django.core.validators.RegexValidator('^[0-9\\-\\(\\)\\ ]+$', message='Enter a valid number')]),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='contact_number',
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(blank=True, help_text='Provide a contact number if repeating glucose measures (Encryption: RSA local)', max_length=71, null=True, validators=[django.core.validators.RegexValidator('^[0-9\\-\\(\\)\\ ]+$', message='Enter a valid number')]),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='contact_number',
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(blank=True, help_text='Provide a contact number if repeating glucose measures (Encryption: RSA local)', max_length=71, null=True, validators=[django.core.validators.RegexValidator('^[0-9\\-\\(\\)\\ ]+$', message='Enter a valid number')]),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='contact_number',
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(blank=True, help_text='Provide a contact number if repeating glucose measures (Encryption: RSA local)', max_length=71, null=True, validators=[django.core.validators.RegexValidator('^[0-9\\-\\(\\)\\ ]+$', message='Enter a valid number')]),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='contact_number',
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(blank=True, help_text='Provide a contact number if repeating glucose measures (Encryption: RSA local)', max_length=71, null=True, validators=[django.core.validators.RegexValidator('^[0-9\\-\\(\\)\\ ]+$', message='Enter a valid number')]),
        ),
    ]



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from django.db import models, transaction
from edc_constants.constants import NEW
from edc_crf.crf_with_action_model_mixin import CrfWithActionModelMixin
from edc_lab.model_mixins import CrfWithRequisitionModelMixin
from edc_lab_panel.panels import rft_panel
from edc_lab_results import BLOOD_RESULTS_RFT_ACTION
from edc_lab_results.model_mixins import (
    BloodResultsModelMixin,
    CreatinineModelMixin,
    EgfrModelMixin,
    UreaModelMixin,
    UricAcidModelMixin,
)
from edc_model import models as edc_models
from edc_reportable import EgfrCkdEpi
from edc_reportable.calculators import egfr_percent_change
from edc_reportable.units import EGFR_UNITS
from edc_screening.utils import get_subject_screening_model_cls
from edc_visit_schedule.constants import DAY1
from edc_visit_schedule.utils import is_baseline


class BloodResultsRft(
    CrfWithActionModelMixin,
    CreatinineModelMixin,
    EgfrModelMixin,
    UreaModelMixin,
    UricAcidModelMixin,
    CrfWithRequisitionModelMixin,
    BloodResultsModelMixin,
    edc_models.BaseUuidModel,
):
    action_name = BLOOD_RESULTS_RFT_ACTION
    tracking_identifier_prefix = "RF"
    lab_panel = rft_panel

    egfr_percent_change = models.DecimalField(
        verbose_name="Percent change from baseline",
        max_digits=10,
        decimal_places=4,
        null=True,
        blank=True,
    )

    def save(self, *args, **kwargs):
        if self.creatinine_value:
            subject_screening = get_subject_screening_model_cls().objects.get(
                subject_identifier=self.subject_visit.subject_identifier
            )
            self.egfr_value = EgfrCkdEpi(
                gender=subject_screening.gender,
                ethnicity=subject_screening.ethnicity,
                age_in_years=subject_screening.age_in_years,
                creatinine_value=self.creatinine_value,
                creatinine_units=self.creatinine_units,
            ).value
            self.egfr_units = EGFR_UNITS
            if is_baseline(self):
                self.egfr_percent_change = 0.0
            else:
                subject_identifier = self.subject_visit.subject_identifier
                baseline_obj = self.__class__.objects.get(
                    subject_visit__subject_identifier=subject_identifier,
                    subject_visit__visit_code=DAY1,
                    subject_visit__visit_code_sequence=0,
                )
                self.egfr_percent_change = egfr_percent_change(
                    float(self.egfr_value), float(baseline_obj.egfr_value)
                )
                if self.egfr_percent_change >= 0.20:
                    self.create_or_update_egfr_notification()
        super().save(*args, **kwargs)

    def create_or_update_egfr_notification(self):
        model_cls = django_apps.get_model("meta_subject.egfrnotification")
        with transaction.atomic():
            try:
                obj = model_cls.objects.get(subject_visit=self.subject_visit)
            except ObjectDoesNotExist:
                model_cls.objects.create(
                    subject_visit=self.subject_visit,
                    report_datetime=self.report_datetime,
                    creatinine_date=self.assay_datetime.date(),
                    egfr_percent_change=self.egfr_percent_change,
                    report_status=NEW,
                    consent_version=self.consent_version,
                )
            else:
                obj.egfr_percent_change = self.egfr_percent_change
                obj.creatinine_date = self.assay_datetime.date()
                obj.save()
            # finally:
            #     if obj.report_status != CLOSED:
            #         obj.action_item.status = OPEN
            #         obj.save()

    class Meta(CrfWithActionModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Blood Result: RFT"
        verbose_name_plural = "Blood Results: RFT"



from django.contrib import admin
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import meta_prn_admin
from ..models import OnSchedule


@admin.register(OnSchedule, site=meta_prn_admin)
class OnScheduleAdmin(
    DataManagerModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):

    instructions = None

    fields = ("subject_identifier", "onschedule_datetime")

    list_display = ("subject_identifier", "dashboard", "onschedule_datetime")

    list_filter = ("onschedule_datetime",)

    def get_readonly_fields(self, request, obj=None):
        return ["subject_identifier", "onschedule_datetime"]



from textwrap import wrap

import arrow
from django import template
from django.conf import settings
from django.utils.safestring import mark_safe
from edc_constants.constants import OTHER, YES
from edc_dashboard.utils import get_bootstrap_version

register = template.Library()


format_ae_description_template_name = (
    f"{settings.ADVERSE_EVENT_APP_LABEL}/bootstrap{get_bootstrap_version()}/"
    f"ae_initial_description.html"
)


@register.inclusion_tag(format_ae_description_template_name, takes_context=True)
def format_ae_description(context, ae_initial, wrap_length):
    context["utc_date"] = arrow.now().date()
    context["SHORT_DATE_FORMAT"] = settings.SHORT_DATE_FORMAT
    context["OTHER"] = OTHER
    context["YES"] = YES
    context["ae_initial"] = ae_initial
    context["sae_reason"] = mark_safe(
        "<BR>".join(wrap(ae_initial.sae_reason.name, wrap_length or 35))
    )
    context["ae_description"] = mark_safe(
        "<BR>".join(wrap(ae_initial.ae_description, wrap_length or 35))
    )
    return context



# Generated by Django 3.2.13 on 2022-07-04 15:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_subject', '0120_alter_birthoutcomes_managers_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicalbirthoutcomes',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Birth Outcomes', 'verbose_name_plural': 'historical Birth Outcomes'},
        ),
        migrations.AlterModelOptions(
            name='historicalbloodresultsfbc',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Blood Result: FBC', 'verbose_name_plural': 'historical Blood Results: FBC'},
        ),
        migrations.AlterModelOptions(
            name='historicalbloodresultsglu',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Blood Result: Glucose', 'verbose_name_plural': 'historical Blood Results: Glucose'},
        ),
        migrations.AlterModelOptions(
            name='historicalbloodresultshba1c',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Blood Result: HbA1c', 'verbose_name_plural': 'historical Blood Results: HbA1c'},
        ),
        migrations.AlterModelOptions(
            name='historicalbloodresultsins',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Blood Result: Insulin', 'verbose_name_plural': 'historical Blood Results: Insulin'},
        ),
        migrations.AlterModelOptions(
            name='historicalbloodresultslft',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Blood Result: LFT', 'verbose_name_plural': 'historical Blood Results: LFT'},
        ),
        migrations.AlterModelOptions(
            name='historicalbloodresultslipid',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Blood Result: Lipids', 'verbose_name_plural': 'historical Blood Results: Lipids'},
        ),
        migrations.AlterModelOptions(
            name='historicalbloodresultsrft',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Blood Result: RFT', 'verbose_name_plural': 'historical Blood Results: RFT'},
        ),
        migrations.AlterModelOptions(
            name='historicalcomplications',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Presence of Complications', 'verbose_name_plural': 'historical Presence of Complications'},
        ),
        migrations.AlterModelOptions(
            name='historicalcomplicationsglycemia',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Complications (Raised Glycemia)', 'verbose_name_plural': 'historical Complications (Raised Glycemia)'},
        ),
        migrations.AlterModelOptions(
            name='historicalconcomitantmedication',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Concomitant Medication', 'verbose_name_plural': 'historical Concomitant Medication'},
        ),
        migrations.AlterModelOptions(
            name='historicaldelivery',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Delivery', 'verbose_name_plural': 'historical Delivery'},
        ),
        migrations.AlterModelOptions(
            name='historicaldietandlifestyle',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Clinic follow up: Examination', 'verbose_name_plural': 'historical Clinic follow up: Examination'},
        ),
        migrations.AlterModelOptions(
            name='historicalegfrnotification',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical eGFR Notification', 'verbose_name_plural': 'historical eGFR Notifications'},
        ),
        migrations.AlterModelOptions(
            name='historicaleq5d3l',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical EuroQol EQ-5D-3L Instrument', 'verbose_name_plural': 'historical EuroQol EQ-5D-3L Instrument'},
        ),
        migrations.AlterModelOptions(
            name='historicalfollowupexamination',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Clinic follow up: Examination', 'verbose_name_plural': 'historical Clinic follow up: Examination'},
        ),
        migrations.AlterModelOptions(
            name='historicalfollowupvitals',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Clinic follow up: Vitals', 'verbose_name_plural': 'historical Clinic follow ups: Vitals'},
        ),
        migrations.AlterModelOptions(
            name='historicalglucose',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Glucose (IFG, OGTT)', 'verbose_name_plural': 'historical Glucose (IFG, OGTT)'},
        ),
        migrations.AlterModelOptions(
            name='historicalhealtheconomics',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Health Economics (M2)', 'verbose_name_plural': 'historical Health Economics (M2)'},
        ),
        migrations.AlterModelOptions(
            name='historicalhealtheconomicssimple',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Health Economics (M3)', 'verbose_name_plural': 'historical Health Economics (M3)'},
        ),
        migrations.AlterModelOptions(
            name='historicalhepatitistest',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Hepatitis Tests', 'verbose_name_plural': 'historical Hepatitis Tests'},
        ),
        migrations.AlterModelOptions(
            name='historicalmalariatest',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Malaria Test', 'verbose_name_plural': 'historical Malaria Tests'},
        ),
        migrations.AlterModelOptions(
            name='historicalmedicationadherence',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Medication Adherence', 'verbose_name_plural': 'historical Medication Adherence'},
        ),
        migrations.AlterModelOptions(
            name='historicalmnsi',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Michigan Neuropathy Screening Instrument (MNSI)', 'verbose_name_plural': 'historical Michigan Neuropathy Screening Instrument (MNSI)'},
        ),
        migrations.AlterModelOptions(
            name='historicalotherarvregimens',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Other ARV Regimens', 'verbose_name_plural': 'historical Other ARV Regimens'},
        ),
        migrations.AlterModelOptions(
            name='historicalotherarvregimensdetail',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Other ARV Regimens Detail', 'verbose_name_plural': 'historical Other ARV Regimens Detail'},
        ),
        migrations.AlterModelOptions(
            name='historicalpatienthistory',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Patient History', 'verbose_name_plural': 'historical Patient History'},
        ),
        migrations.AlterModelOptions(
            name='historicalphysicalexam',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Physical Exam', 'verbose_name_plural': 'historical Physical Exams'},
        ),
        migrations.AlterModelOptions(
            name='historicalpregnancyupdate',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Pregnancy Update', 'verbose_name_plural': 'historical Pregnancy Update'},
        ),
        migrations.AlterModelOptions(
            name='historicalsf12',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical SF-12 Health Survey', 'verbose_name_plural': 'historical SF-12 Health Survey'},
        ),
        migrations.AlterModelOptions(
            name='historicalstudymedication',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Study Medication', 'verbose_name_plural': 'historical Study Medication'},
        ),
        migrations.AlterModelOptions(
            name='historicalsubjectrequisition',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical subject requisition', 'verbose_name_plural': 'historical subject requisitions'},
        ),
        migrations.AlterModelOptions(
            name='historicalsubjectvisit',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical subject visit', 'verbose_name_plural': 'historical subject visits'},
        ),
        migrations.AlterModelOptions(
            name='historicalsubjectvisitmissed',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Missed Visit Report', 'verbose_name_plural': 'historical Missed Visit Report'},
        ),
        migrations.AlterModelOptions(
            name='historicalurinedipsticktest',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Urine Dipstick Test', 'verbose_name_plural': 'historical Urine Dipstick Tests'},
        ),
        migrations.AlterModelOptions(
            name='historicalurinepregnancy',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Urine Pregnancy', 'verbose_name_plural': 'historical Urine Pregnancys'},
        ),
        migrations.AddField(
            model_name='historicalmedicationadherence',
            name='pill_count_not_performed_reason',
            field=models.TextField(blank=True, null=True, verbose_name='Was a pill count performed'),
        ),
        migrations.AddField(
            model_name='medicationadherence',
            name='pill_count_not_performed_reason',
            field=models.TextField(blank=True, null=True, verbose_name='Was a pill count performed'),
        ),
        migrations.AlterField(
            model_name='historicalbirthoutcomes',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalbloodresultsfbc',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalbloodresultsglu',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalbloodresultshba1c',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalbloodresultsins',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalbloodresultslft',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalbloodresultslipid',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalbloodresultsrft',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalcomplications',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalcomplicationsglycemia',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalconcomitantmedication',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicaldelivery',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicaldietandlifestyle',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalegfrnotification',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicaleq5d3l',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalfollowupexamination',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalfollowupvitals',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalglucose',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalhealtheconomics',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalhealtheconomicssimple',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalhepatitistest',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalmalariatest',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalmedicationadherence',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalmnsi',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalotherarvregimens',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalotherarvregimensdetail',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalpatienthistory',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalphysicalexam',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalpregnancyupdate',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalsf12',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalstudymedication',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalsubjectrequisition',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalsubjectvisit',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalsubjectvisitmissed',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalurinedipsticktest',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalurinepregnancy',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
    ]



from django.contrib import admin
from django.db.models import Q
from django.utils.translation import gettext as _
from edc_appointment.admin import AppointmentListFilter
from edc_constants.constants import NO, NOT_APPLICABLE, PENDING, TBD, YES

from meta_screening.constants import PENDING_REPEAT

NEXT_MONTH = "next_month"
NEXT_WEEK = "next_week"
THIS_WEEK = "this_week"
TODAY = "today"
LAST_WEEK = "last_week"
LAST_MONTH = "last_month"
NOT_NULL = "not_null"
PAST_DATE = "past_date"
FUTURE_DATE = "future_date"


class EligibilityPending(admin.SimpleListFilter):
    title = _("P3 pending")

    parameter_name = "eligibility_pending"

    def lookups(self, request, model_admin):
        return (
            (PENDING, "Pending"),
            (PENDING_REPEAT, "Pending repeat GLU"),
        )

    def queryset(self, request, queryset):
        if self.value() == PENDING_REPEAT:
            return queryset.filter(
                eligible_part_one=YES,
                eligible_part_two=YES,
                eligible_part_three=TBD,
                part_three_report_datetime__isnull=False,
                repeat_glucose_opinion=YES,
            )
        if self.value() == PENDING:
            return queryset.filter(
                eligible_part_one=YES,
                eligible_part_two=YES,
                eligible_part_three=TBD,
                agree_to_p3=YES,
                p3_ltfu=NOT_APPLICABLE,
                part_three_report_datetime__isnull=True,
            )


class P3LtfuListFilter(admin.SimpleListFilter):
    title = _("Contact for P3?")

    parameter_name = "p3_ltfu_custom"

    def lookups(self, request, model_admin):
        return (
            (YES, "Yes"),
            (NO, "No, lost contact"),
        )

    def queryset(self, request, queryset):
        if self.value() == YES:
            return queryset.filter(
                eligible_part_one=YES,
                eligible_part_two=YES,
                eligible_part_three=TBD,
                p3_ltfu=NOT_APPLICABLE,
            )
        if self.value() == NO:
            return queryset.filter(
                eligible_part_one=YES,
                eligible_part_two=YES,
                eligible_part_three=TBD,
                p3_ltfu=YES,
            )
        if self.value() == NO:
            return queryset.exclude(
                Q(eligible_part_one=TBD)
                | Q(eligible_part_two=TBD)
                | Q(eligible_part_three=TBD)
            )


class P3ApptListFilter(AppointmentListFilter):
    title = _("Part 3 appointment")
    parameter_name = "p3_appt_datetime"
    field_name = "appt_datetime"

    @property
    def extra_queryset_options(self):
        return dict(
            eligible_part_one=YES,
            eligible_part_two=YES,
            eligible_part_three=TBD,
        )



# Generated by Django 4.0.5 on 2022-06-25 18:56

from django.db import migrations, models
import django.db.models.deletion
import edc_model.validators.date


class Migration(migrations.Migration):

    dependencies = [
        ('edc_protocol_violation', '0005_protocolincident_historicalprotocolincident_and_more'),
        ('meta_prn', '0025_alter_historicalprotocoldeviationviolation_options_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalprotocoldeviationviolation',
            name='violation_type',
        ),
        migrations.RemoveField(
            model_name='historicalprotocoldeviationviolation',
            name='violation_type_other',
        ),
        migrations.RemoveField(
            model_name='protocoldeviationviolation',
            name='violation_type',
        ),
        migrations.RemoveField(
            model_name='protocoldeviationviolation',
            name='violation_type_other',
        ),
        migrations.AlterField(
            model_name='historicalprotocoldeviationviolation',
            name='incident',
            field=models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='edc_protocol_violation.protocolviolations', verbose_name='Type of incident'),
        ),
        migrations.AlterField(
            model_name='historicalprotocoldeviationviolation',
            name='incident_datetime',
            field=models.DateTimeField(null=True, validators=[edc_model.validators.date.datetime_not_future], verbose_name='Date incident occurred'),
        ),
        migrations.AlterField(
            model_name='historicalprotocoldeviationviolation',
            name='incident_description',
            field=models.TextField(help_text='Describe in full. Explain how the incident happened, what occurred, etc.', null=True, verbose_name='Describe the incident'),
        ),
        migrations.AlterField(
            model_name='historicalprotocoldeviationviolation',
            name='incident_reason',
            field=models.TextField(null=True, verbose_name='Explain the reason why the incident occurred'),
        ),
        migrations.AlterField(
            model_name='protocoldeviationviolation',
            name='incident',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_protocol_violation.protocolviolations', verbose_name='Type of incident'),
        ),
        migrations.AlterField(
            model_name='protocoldeviationviolation',
            name='incident_datetime',
            field=models.DateTimeField(null=True, validators=[edc_model.validators.date.datetime_not_future], verbose_name='Date incident occurred'),
        ),
        migrations.AlterField(
            model_name='protocoldeviationviolation',
            name='incident_description',
            field=models.TextField(help_text='Describe in full. Explain how the incident happened, what occurred, etc.', null=True, verbose_name='Describe the incident'),
        ),
        migrations.AlterField(
            model_name='protocoldeviationviolation',
            name='incident_reason',
            field=models.TextField(null=True, verbose_name='Explain the reason why the incident occurred'),
        ),
    ]



# Generated by Django 3.2.13 on 2022-07-04 15:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0042_remove_endofstudy_investigator_decision_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicalendofstudy',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical End of Study', 'verbose_name_plural': 'historical End of Study'},
        ),
        migrations.AlterModelOptions(
            name='historicallosstofollowup',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Loss to Follow Up', 'verbose_name_plural': 'historical Loss to Follow Up'},
        ),
        migrations.AlterModelOptions(
            name='historicaloffschedule',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Off-schedule', 'verbose_name_plural': 'historical Off-schedule'},
        ),
        migrations.AlterModelOptions(
            name='historicaloffschedulepostnatal',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Off-schedule: post-natal', 'verbose_name_plural': 'historical Off-schedule: post-natal'},
        ),
        migrations.AlterModelOptions(
            name='historicaloffschedulepregnancy',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Off-schedule: Pregnancy', 'verbose_name_plural': 'historical Off-schedule: Pregnancy'},
        ),
        migrations.AlterModelOptions(
            name='historicaloffstudymedication',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Withdrawal of Study Drug', 'verbose_name_plural': 'historical Withdrawal of Study Drug'},
        ),
        migrations.AlterModelOptions(
            name='historicalonschedule',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical on schedule', 'verbose_name_plural': 'historical on schedules'},
        ),
        migrations.AlterModelOptions(
            name='historicalonschedulepostnatal',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical on schedule postnatal', 'verbose_name_plural': 'historical on schedule postnatals'},
        ),
        migrations.AlterModelOptions(
            name='historicalonschedulepregnancy',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical on schedule pregnancy', 'verbose_name_plural': 'historical on schedule pregnancys'},
        ),
        migrations.AlterModelOptions(
            name='historicalpregnancynotification',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Pregnancy Notification', 'verbose_name_plural': 'historical Pregnancy Notifications'},
        ),
        migrations.AlterModelOptions(
            name='historicalprotocolincident',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Protocol Incident', 'verbose_name_plural': 'historical Protocol Incident'},
        ),
        migrations.AlterModelOptions(
            name='historicalsubjecttransfer',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Subject Transfer', 'verbose_name_plural': 'historical Subject Transfers'},
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicallosstofollowup',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicaloffschedule',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicaloffschedulepostnatal',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicaloffschedulepregnancy',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicaloffstudymedication',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalonschedule',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalonschedulepostnatal',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalonschedulepregnancy',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalpregnancynotification',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalprotocolincident',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalsubjecttransfer',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
    ]



from django import forms
from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_constants.constants import CLOSED, NEW, OPEN
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import INVALID_ERROR
from edc_form_validators.form_validator import FormValidator

from ..models import EgfrNotification


class EgfrNotificationFormValidator(FormValidator):
    def clean(self):
        self.required_if(OPEN, CLOSED, field="report_status", field_required="narrative")
        if self.cleaned_data.get("report_status") == NEW:
            self.raise_validation_error(
                {"report_status": "Cannot be NEW, set to OPEN or CLOSED"}, INVALID_ERROR
            )


class EgfrNotificationForm(CrfModelFormMixin, ActionItemFormMixin, forms.ModelForm):

    form_validator_cls = EgfrNotificationFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = EgfrNotification
        fields = "__all__"



# Generated by Django 4.0.5 on 2022-06-29 17:26

import django.contrib.sites.managers
from django.db import migrations, models
import edc_visit_tracking.managers


class Migration(migrations.Migration):

    dependencies = [
        ('meta_subject', '0116_egfrnotification_report_status_and_more'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='egfrnotification',
            managers=[
                ('on_site', django.contrib.sites.managers.CurrentSiteManager()),
                ('objects', edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AddField(
            model_name='egfrnotification',
            name='crf_status',
            field=models.CharField(choices=[('INCOMPLETE', 'Incomplete (some data pending)'), ('COMPLETE', 'Complete')], default='COMPLETE', help_text='If some data is still pending, flag this CRF as incomplete', max_length=25, verbose_name='CRF status'),
        ),
        migrations.AddField(
            model_name='egfrnotification',
            name='crf_status_comments',
            field=models.TextField(blank=True, help_text='for example, why some data is still pending', null=True, verbose_name='Any comments related to status of this CRF'),
        ),
        migrations.AddField(
            model_name='historicalegfrnotification',
            name='crf_status',
            field=models.CharField(choices=[('INCOMPLETE', 'Incomplete (some data pending)'), ('COMPLETE', 'Complete')], default='COMPLETE', help_text='If some data is still pending, flag this CRF as incomplete', max_length=25, verbose_name='CRF status'),
        ),
        migrations.AddField(
            model_name='historicalegfrnotification',
            name='crf_status_comments',
            field=models.TextField(blank=True, help_text='for example, why some data is still pending', null=True, verbose_name='Any comments related to status of this CRF'),
        ),
        migrations.AlterField(
            model_name='egfrnotification',
            name='egfr_percent_change',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='Copied from RFT result eGFR section', max_digits=10, null=True, verbose_name='Change from baseline'),
        ),
        migrations.AlterField(
            model_name='historicalegfrnotification',
            name='egfr_percent_change',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='Copied from RFT result eGFR section', max_digits=10, null=True, verbose_name='Change from baseline'),
        ),
    ]



# Generated by Django 4.0.5 on 2022-07-03 13:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0039_historicaloffstudymedication_reason_other_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicaloffstudymedication',
            name='expected_restart_date',
        ),
        migrations.RemoveField(
            model_name='historicaloffstudymedication',
            name='permanently_discontinued',
        ),
        migrations.RemoveField(
            model_name='historicaloffstudymedication',
            name='permanently_discontinued_reason',
        ),
        migrations.RemoveField(
            model_name='offstudymedication',
            name='expected_restart_date',
        ),
        migrations.RemoveField(
            model_name='offstudymedication',
            name='permanently_discontinued',
        ),
        migrations.RemoveField(
            model_name='offstudymedication',
            name='permanently_discontinued_reason',
        ),
        migrations.AddField(
            model_name='endofstudy',
            name='investigator_decision',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name='If reason is `investigator decision`, please explain ...'),
        ),
        migrations.AddField(
            model_name='historicalendofstudy',
            name='investigator_decision',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name='If reason is `investigator decision`, please explain ...'),
        ),
        migrations.AddField(
            model_name='historicaloffstudymedication',
            name='comment',
            field=models.TextField(blank=True, null=True, verbose_name='Comment'),
        ),
        migrations.AddField(
            model_name='offstudymedication',
            name='comment',
            field=models.TextField(blank=True, null=True, verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='clinical_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('diabetes', 'Patient developed diabetes'), ('kidney_disease', 'Development of chronic kidney disease'), ('liver_disease', 'Development of chronic liver disease'), ('toxicity', 'Patient experienced an unacceptable toxicity'), ('intercurrent_illness', 'Intercurrent illness which prevents further treatment'), ('investigator_decision', 'Investigator decision (specify below)'), ('OTHER', 'Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)')], max_length=25, null=True, verbose_name='If withdrawn on CLINICAL grounds, please specify PRIMARY reason...'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='clinical_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('diabetes', 'Patient developed diabetes'), ('kidney_disease', 'Development of chronic kidney disease'), ('liver_disease', 'Development of chronic liver disease'), ('toxicity', 'Patient experienced an unacceptable toxicity'), ('intercurrent_illness', 'Intercurrent illness which prevents further treatment'), ('investigator_decision', 'Investigator decision (specify below)'), ('OTHER', 'Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)')], max_length=25, null=True, verbose_name='If withdrawn on CLINICAL grounds, please specify PRIMARY reason...'),
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-27 14:17

from django.db import migrations
import django_crypto_fields.fields.encrypted_char_field


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0044_alter_historicalscreeningpartone_severe_htn_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='contact_number',
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(blank=True, help_text=' (Encryption: RSA local)', max_length=71, null=True),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='contact_number',
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(blank=True, help_text=' (Encryption: RSA local)', max_length=71, null=True),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='contact_number',
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(blank=True, help_text=' (Encryption: RSA local)', max_length=71, null=True),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='contact_number',
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(blank=True, help_text=' (Encryption: RSA local)', max_length=71, null=True),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='contact_number',
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(blank=True, help_text=' (Encryption: RSA local)', max_length=71, null=True),
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-28 11:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0049_historicalscreeningpartone_p3_ltfu_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='agree_to_p3',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable, subject is not eligible based on the criteria above')], default='Yes', max_length=15, verbose_name='Has the patient agreed to complete/return for the second stage of the screening?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='agree_to_p3',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable, subject is not eligible based on the criteria above')], default='Yes', max_length=15, verbose_name='Has the patient agreed to complete/return for the second stage of the screening?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='agree_to_p3',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable, subject is not eligible based on the criteria above')], default='Yes', max_length=15, verbose_name='Has the patient agreed to complete/return for the second stage of the screening?'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='agree_to_p3',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable, subject is not eligible based on the criteria above')], default='Yes', max_length=15, verbose_name='Has the patient agreed to complete/return for the second stage of the screening?'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='agree_to_p3',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable, subject is not eligible based on the criteria above')], default='Yes', max_length=15, verbose_name='Has the patient agreed to complete/return for the second stage of the screening?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='advised_to_fast',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Has the patient been advised to return FASTED for the second stage of the screening?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='p3_ltfu',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Undecided / Not applicable')], default='N/A', help_text='Only applicable if the subject missed the appointment for the second stage of screening. See above', max_length=15, verbose_name='Should additional attempts be made to contact this subject'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='p3_ltfu_comment',
            field=models.TextField(blank=True, null=True, verbose_name='Provide any additional comments on this decision (or leave blank)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='p3_ltfu_date',
            field=models.DateField(blank=True, help_text='Must be after the appointment date for the second stage of screening (P3)', null=True, verbose_name='Date decision made'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='advised_to_fast',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Has the patient been advised to return FASTED for the second stage of the screening?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='p3_ltfu',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Undecided / Not applicable')], default='N/A', help_text='Only applicable if the subject missed the appointment for the second stage of screening. See above', max_length=15, verbose_name='Should additional attempts be made to contact this subject'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='p3_ltfu_comment',
            field=models.TextField(blank=True, null=True, verbose_name='Provide any additional comments on this decision (or leave blank)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='p3_ltfu_date',
            field=models.DateField(blank=True, help_text='Must be after the appointment date for the second stage of screening (P3)', null=True, verbose_name='Date decision made'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='advised_to_fast',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Has the patient been advised to return FASTED for the second stage of the screening?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='p3_ltfu',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Undecided / Not applicable')], default='N/A', help_text='Only applicable if the subject missed the appointment for the second stage of screening. See above', max_length=15, verbose_name='Should additional attempts be made to contact this subject'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='p3_ltfu_comment',
            field=models.TextField(blank=True, null=True, verbose_name='Provide any additional comments on this decision (or leave blank)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='p3_ltfu_date',
            field=models.DateField(blank=True, help_text='Must be after the appointment date for the second stage of screening (P3)', null=True, verbose_name='Date decision made'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='advised_to_fast',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Has the patient been advised to return FASTED for the second stage of the screening?'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='p3_ltfu',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Undecided / Not applicable')], default='N/A', help_text='Only applicable if the subject missed the appointment for the second stage of screening. See above', max_length=15, verbose_name='Should additional attempts be made to contact this subject'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='p3_ltfu_comment',
            field=models.TextField(blank=True, null=True, verbose_name='Provide any additional comments on this decision (or leave blank)'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='p3_ltfu_date',
            field=models.DateField(blank=True, help_text='Must be after the appointment date for the second stage of screening (P3)', null=True, verbose_name='Date decision made'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='advised_to_fast',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', max_length=15, verbose_name='Has the patient been advised to return FASTED for the second stage of the screening?'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='p3_ltfu',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Undecided / Not applicable')], default='N/A', help_text='Only applicable if the subject missed the appointment for the second stage of screening. See above', max_length=15, verbose_name='Should additional attempts be made to contact this subject'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='p3_ltfu_comment',
            field=models.TextField(blank=True, null=True, verbose_name='Provide any additional comments on this decision (or leave blank)'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='p3_ltfu_date',
            field=models.DateField(blank=True, help_text='Must be after the appointment date for the second stage of screening (P3)', null=True, verbose_name='Date decision made'),
        ),
    ]



from edc_protocol_violation.forms import ProtocolIncidentForm as Base

from ..models import ProtocolIncident


class ProtocolIncidentForm(Base):
    class Meta:
        model = ProtocolIncident
        fields = "__all__"



# Generated by Django 4.0.5 on 2022-07-03 19:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0041_endofstudy_transfer_date_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='endofstudy',
            name='investigator_decision',
        ),
        migrations.RemoveField(
            model_name='historicalendofstudy',
            name='investigator_decision',
        ),
        migrations.AddField(
            model_name='endofstudy',
            name='clinical_withdrawal_investigator_decision',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name="If withdrawl was an 'investigator decision', please explain ..."),
        ),
        migrations.AddField(
            model_name='historicalendofstudy',
            name='clinical_withdrawal_investigator_decision',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name="If withdrawl was an 'investigator decision', please explain ..."),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='clinical_withdrawal_reason',
            field=models.CharField(choices=[('kidney_disease', 'Development of chronic kidney disease'), ('liver_disease', 'Development of chronic liver disease'), ('intercurrent_illness', 'Intercurrent illness which prevents further treatment'), ('investigator_decision', 'Investigator decision (specify below)'), ('OTHER', 'Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name='If the patient was withdrawn on CLINICAL grounds, please specify PRIMARY reason'),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='clinical_withdrawal_reason_other',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name="If withdrawn for 'other' condition, please explain"),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='toxicity_withdrawal_reason',
            field=models.CharField(choices=[('lactic_acidosis', 'Development of lactic acidosis or hyperlactatemia'), ('hepatomegaly', 'Development of hepatomegaly with steatosis'), ('OTHER', 'Other toxicity (specify below)'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name=" If the patient experienced an unacceptable toxicity', please explain"),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='toxicity_withdrawal_reason_other',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name="If 'other toxicity', please specify ..."),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='clinical_withdrawal_reason',
            field=models.CharField(choices=[('kidney_disease', 'Development of chronic kidney disease'), ('liver_disease', 'Development of chronic liver disease'), ('intercurrent_illness', 'Intercurrent illness which prevents further treatment'), ('investigator_decision', 'Investigator decision (specify below)'), ('OTHER', 'Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name='If the patient was withdrawn on CLINICAL grounds, please specify PRIMARY reason'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='clinical_withdrawal_reason_other',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name="If withdrawn for 'other' condition, please explain"),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='toxicity_withdrawal_reason',
            field=models.CharField(choices=[('lactic_acidosis', 'Development of lactic acidosis or hyperlactatemia'), ('hepatomegaly', 'Development of hepatomegaly with steatosis'), ('OTHER', 'Other toxicity (specify below)'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name=" If the patient experienced an unacceptable toxicity', please explain"),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='toxicity_withdrawal_reason_other',
            field=models.TextField(blank=True, max_length=500, null=True, verbose_name="If 'other toxicity', please specify ..."),
        ),
    ]



from copy import copy

from django.contrib import admin
from django.utils.html import format_html
from django_audit_fields import audit_fieldset_tuple
from edc_action_item import action_fields, action_fieldset_tuple
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import meta_prn_admin
from ..forms import OffScheduleForm
from ..models import EndOfStudy, OffSchedule


@admin.register(OffSchedule, site=meta_prn_admin)
class OffScheduleAdmin(
    DataManagerModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):

    form = OffScheduleForm

    additional_instructions = format_html(
        '<span style="color:orange;font-weight:bold">Note:</span> Detailed '
        "information about study termination will be asked for on the "
        f"<b>{EndOfStudy._meta.verbose_name}</b> form"
    )

    fieldsets = (
        (None, {"fields": ("subject_identifier", "offschedule_datetime")}),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = ("subject_identifier", "dashboard", "offschedule_datetime")

    list_filter = ("offschedule_datetime",)

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        fields = list(action_flds) + list(fields)
        return fields



from django.db import models
from edc_constants.constants import COMPLETE, INCOMPLETE, NEW, OPEN
from edc_crf.crf_status_model_mixin import CrfStatusModelMixin
from edc_crf.crf_with_action_model_mixin import CrfWithActionModelMixin
from edc_model import models as edc_models
from edc_utils import get_utcnow

from meta_screening.models.creatinine_fields_model_mixin import (
    CreatinineModelFieldsMixin,
)

from ..choices import REPORT_STATUS
from ..constants import EGFR_NOTIFICATION_ACTION


class EgfrNotification(
    CrfStatusModelMixin,
    CrfWithActionModelMixin,
    CreatinineModelFieldsMixin,
    edc_models.BaseUuidModel,
):

    action_name = EGFR_NOTIFICATION_ACTION

    tracking_identifier_prefix = "EG"

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=get_utcnow
    )

    creatinine_date = models.DateField(verbose_name="Creatinine result date")

    egfr_percent_change = models.DecimalField(
        verbose_name="Change from baseline",
        max_digits=10,
        decimal_places=2,
        null=True,
        blank=True,
        help_text="Copied from RFT result eGFR section",
    )

    narrative = models.TextField(
        verbose_name="Narrative",
        null=True,
        blank=True,
    )

    report_status = models.CharField(max_length=15, choices=REPORT_STATUS, default=NEW)

    def save(self, *args, **kwargs):
        if self.report_status == OPEN:
            self.crf_status = INCOMPLETE
        else:
            self.crf_status = COMPLETE
        super().save(*args, **kwargs)

    class Meta(edc_models.BaseUuidModel.Meta):
        verbose_name = "eGFR Notification"
        verbose_name_plural = "eGFR Notifications"



# Generated by Django 4.0.5 on 2022-07-03 09:35

from django.db import migrations, models
import django.db.models.deletion
import edc_model.validators.date


class Migration(migrations.Migration):

    dependencies = [
        ('meta_lists', '0012_auto_20210728_1809'),
        ('meta_prn', '0036_remove_endofstudy_meta_prn_en_id_a50384_idx_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='endofstudy',
            name='delivery_date',
            field=models.DateField(blank=True, help_text='A delivery CRF must be on file', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date of delivery, if applicable'),
        ),
        migrations.AddField(
            model_name='endofstudy',
            name='pregnancy_date',
            field=models.DateField(blank=True, help_text='A UPT CRF must be on file. Use UPT date or, if UPT not done, use report date on last UPT CRF.', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date pregnancy reported/UPT, if applicable'),
        ),
        migrations.AddField(
            model_name='historicalendofstudy',
            name='delivery_date',
            field=models.DateField(blank=True, help_text='A delivery CRF must be on file', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date of delivery, if applicable'),
        ),
        migrations.AddField(
            model_name='historicalendofstudy',
            name='pregnancy_date',
            field=models.DateField(blank=True, help_text='A UPT CRF must be on file. Use UPT date or, if UPT not done, use report date on last UPT CRF.', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date pregnancy reported/UPT, if applicable'),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='clinical_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('diabetes', 'Patient developed diabetes'), ('kidney_disease', 'Development of chronic kidney disease'), ('liver_disease', 'Development of chronic liver disease'), ('toxicity', 'Patient experienced an unacceptable toxicity'), ('intercurrent_illness', 'Intercurrent illness which prevents further treatment'), ('investigator_decision', 'Investigator decision'), ('OTHER', 'Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)')], max_length=25, null=True, verbose_name='If withdrawn on CLINICAL grounds, please specify PRIMARY reason...'),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='death_date',
            field=models.DateField(blank=True, help_text='A Death report must be on file', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date of death, if applicable'),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='offstudy_reason',
            field=models.ForeignKey(limit_choices_to={'name__in': ['completed_followup', 'delivery', 'pregnancy', 'clinical_withdrawal', 'clinical_endpoint', 'LTFU', 'dead', 'withdrawal', 'late_exclusion', 'transferred', 'OTHER']}, null=True, on_delete=django.db.models.deletion.PROTECT, to='meta_lists.offstudyreasons', verbose_name='Reason patient was terminated from the study'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='clinical_withdrawal_reason',
            field=models.CharField(blank=True, choices=[('diabetes', 'Patient developed diabetes'), ('kidney_disease', 'Development of chronic kidney disease'), ('liver_disease', 'Development of chronic liver disease'), ('toxicity', 'Patient experienced an unacceptable toxicity'), ('intercurrent_illness', 'Intercurrent illness which prevents further treatment'), ('investigator_decision', 'Investigator decision'), ('OTHER', 'Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)')], max_length=25, null=True, verbose_name='If withdrawn on CLINICAL grounds, please specify PRIMARY reason...'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='death_date',
            field=models.DateField(blank=True, help_text='A Death report must be on file', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date of death, if applicable'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='offstudy_reason',
            field=models.ForeignKey(blank=True, db_constraint=False, limit_choices_to={'name__in': ['completed_followup', 'delivery', 'pregnancy', 'clinical_withdrawal', 'clinical_endpoint', 'LTFU', 'dead', 'withdrawal', 'late_exclusion', 'transferred', 'OTHER']}, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='meta_lists.offstudyreasons', verbose_name='Reason patient was terminated from the study'),
        ),
    ]



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_adherence.model_admin_mixin import (
    MedicationAdherenceAdminMixin,
    get_visual_score_fieldset_tuple,
    missed_medications_fieldset_tuple,
)

from ..admin_site import meta_subject_admin
from ..forms import MedicationAdherenceForm
from ..models import MedicationAdherence
from .modeladmin import CrfModelAdmin

pill_count_fieldset_tuple = (
    "Pill Count",
    {
        "fields": ["pill_count_performed", "pill_count_not_performed_reason", "pill_count"],
    },
)


@admin.register(MedicationAdherence, site=meta_subject_admin)
class MedicationAdherenceAdmin(MedicationAdherenceAdminMixin, CrfModelAdmin):

    form = MedicationAdherenceForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        get_visual_score_fieldset_tuple(),
        pill_count_fieldset_tuple,
        missed_medications_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "pill_count_performed": admin.VERTICAL,
        "last_missed_pill": admin.VERTICAL,
    }



# Generated by Django 4.0.5 on 2022-06-24 09:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_subject', '0111_alter_followupvitals_severe_htn_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalsubjectvisit',
            name='document_status_comments',
            field=models.TextField(blank=True, help_text='for example, why some data is still pending', null=True, verbose_name='Any comments related to status of this document'),
        ),
        migrations.AddField(
            model_name='subjectvisit',
            name='document_status_comments',
            field=models.TextField(blank=True, help_text='for example, why some data is still pending', null=True, verbose_name='Any comments related to status of this document'),
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-25 20:57

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edc_protocol_violation', '0006_protocolincidents_and_more'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('edc_action_item', '0028_auto_20210203_0706'),
        ('sites', '0002_alter_domain_unique'),
        ('meta_prn', '0026_remove_historicalprotocoldeviationviolation_violation_type_and_more'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='HistoricalProtocolDeviationViolation',
            new_name='HistoricalProtocolIncident',
        ),
        migrations.RenameModel(
            old_name='ProtocolDeviationViolation',
            new_name='ProtocolIncident',
        ),
        migrations.RemoveIndex(
            model_name='protocolincident',
            name='meta_prn_pr_subject_6d4791_idx',
        ),
        migrations.AddIndex(
            model_name='protocolincident',
            index=models.Index(fields=['subject_identifier', 'action_identifier', 'site', 'id'], name='meta_prn_pr_subject_e85b31_idx'),
        ),
    ]



from django import forms
from edc_action_item.forms import ActionItemFormMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import FormValidator
from edc_lab_panel.panels import hba1c_panel, hba1c_poc_panel
from edc_lab_results.form_validator_mixins import BloodResultsFormValidatorMixin

from ...models import BloodResultsHba1c


class BloodResultsHba1cFormValidator(BloodResultsFormValidatorMixin, FormValidator):
    panels = [hba1c_poc_panel, hba1c_panel]


class BloodResultsHba1cForm(ActionItemFormMixin, CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = BloodResultsHba1cFormValidator

    class Meta:
        model = BloodResultsHba1c
        fields = "__all__"



# Generated by Django 4.0.5 on 2022-06-24 09:34

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('edc_protocol_violation', '0004_alter_protocoldeviationviolation_violation'),
        ('meta_prn', '0023_auto_20220415_1747'),
    ]

    operations = [
        migrations.AlterField(
            model_name='protocoldeviationviolation',
            name='violation',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_protocol_violation.protocolviolations', verbose_name='Type of violation'),
        ),
    ]



# Generated by Django 4.0.5 on 2022-07-03 12:48

import _socket
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_action_item.models.action_model_mixin
import edc_model.validators.date
import edc_utils.date
import simple_history.models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sites', '0002_alter_domain_unique'),
        ('edc_action_item', '0028_auto_20210203_0706'),
        ('meta_prn', '0037_endofstudy_delivery_date_endofstudy_pregnancy_date_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='endofstudy',
            name='delivery_date',
            field=models.DateField(blank=True, help_text='A Delivery CRF must be on file. Use delivery date, if reported, or report date from Delivery CRF', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date of delivery, if applicable'),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='ltfu_date',
            field=models.DateField(blank=True, help_text='A Loss to followup report must be on file', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date lost to followup, if applicable'),
        ),
        migrations.AlterField(
            model_name='endofstudy',
            name='pregnancy_date',
            field=models.DateField(blank=True, help_text='A UPT CRF must be on file and participant not on the delivery schedule. Use UPT date or, if UPT not needed, use report date on last UPT CRF.', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date pregnancy known/UPT, if applicable'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='delivery_date',
            field=models.DateField(blank=True, help_text='A Delivery CRF must be on file. Use delivery date, if reported, or report date from Delivery CRF', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date of delivery, if applicable'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='ltfu_date',
            field=models.DateField(blank=True, help_text='A Loss to followup report must be on file', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date lost to followup, if applicable'),
        ),
        migrations.AlterField(
            model_name='historicalendofstudy',
            name='pregnancy_date',
            field=models.DateField(blank=True, help_text='A UPT CRF must be on file and participant not on the delivery schedule. Use UPT date or, if UPT not needed, use report date on last UPT CRF.', null=True, validators=[edc_model.validators.date.date_not_future], verbose_name='Date pregnancy known/UPT, if applicable'),
        ),
        migrations.CreateModel(
            name='OffStudyMedication',
            fields=[
                ('revision', django_revision.revision_field.RevisionField(blank=True, editable=False, help_text='System field. Git repository tag:branch:commit.', max_length=75, null=True, verbose_name='Revision')),
                ('created', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('modified', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('user_created', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user created')),
                ('user_modified', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user modified')),
                ('hostname_created', models.CharField(blank=True, default=_socket.gethostname, help_text='System field. (modified on create only)', max_length=60)),
                ('hostname_modified', django_audit_fields.fields.hostname_modification_field.HostnameModificationField(blank=True, help_text='System field. (modified on every save)', max_length=50)),
                ('device_created', models.CharField(blank=True, max_length=10)),
                ('device_modified', models.CharField(blank=True, max_length=10)),
                ('id', django_audit_fields.fields.uuid_auto_field.UUIDAutoField(blank=True, editable=False, help_text='System auto field. UUID primary key.', primary_key=True, serialize=False)),
                ('subject_identifier', models.CharField(max_length=50)),
                ('tracking_identifier', models.CharField(max_length=32, unique=True)),
                ('action_identifier', models.CharField(max_length=50, unique=True)),
                ('parent_action_identifier', models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True)),
                ('related_action_identifier', models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True)),
                ('action_item_reason', models.TextField(editable=False, null=True)),
                ('report_datetime', models.DateTimeField(default=edc_utils.date.get_utcnow, verbose_name='Report Date and Time')),
                ('stop_date', models.DateField(verbose_name='Date decision to stop study medication')),
                ('last_dose_date', models.DateField(verbose_name='Date of last known dose')),
                ('reason', models.CharField(choices=[('pregnancy', 'Pregnancy'), ('sae', 'Participant is experiencing a serious adverse event'), ('clinician', 'Other condition that justifies discontinuation of treatment in the clinician’s opinion (specify below)'), ('investigator', ' Investigator decision'), ('patient', 'Patient decision'), ('OTHER', 'Other reason (specify below)')], max_length=25, verbose_name='Reason for stopping study medication')),
                ('permanently_discontinued', models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], max_length=15, verbose_name='Is study medication being permanently discontinued')),
                ('permanently_discontinued_reason', models.TextField(blank=True, null=True, verbose_name='Reason study medication being permanently discontinued')),
                ('expected_restart_date', models.DateField(blank=True, null=True, validators=[edc_model.validators.date.date_is_future], verbose_name='If not permanently discontinued, expected restart date')),
                ('action_item', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='edc_action_item.actionitem')),
                ('parent_action_item', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem')),
                ('related_action_item', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem')),
                ('site', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='sites.site')),
            ],
            options={
                'verbose_name': 'Withdrawal of Study Drug',
                'verbose_name_plural': 'Withdrawal of Study Drug',
                'ordering': ('-modified', '-created'),
                'get_latest_by': 'modified',
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'export', 'import'),
            },
            managers=[
                ('objects', edc_action_item.models.action_model_mixin.ActionItemModelManager()),
            ],
        ),
        migrations.CreateModel(
            name='HistoricalOffStudyMedication',
            fields=[
                ('revision', django_revision.revision_field.RevisionField(blank=True, editable=False, help_text='System field. Git repository tag:branch:commit.', max_length=75, null=True, verbose_name='Revision')),
                ('created', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('modified', models.DateTimeField(blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow)),
                ('user_created', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user created')),
                ('user_modified', django_audit_fields.fields.userfield.UserField(blank=True, help_text='Updated by admin.save_model', max_length=50, verbose_name='user modified')),
                ('hostname_created', models.CharField(blank=True, default=_socket.gethostname, help_text='System field. (modified on create only)', max_length=60)),
                ('hostname_modified', django_audit_fields.fields.hostname_modification_field.HostnameModificationField(blank=True, help_text='System field. (modified on every save)', max_length=50)),
                ('device_created', models.CharField(blank=True, max_length=10)),
                ('device_modified', models.CharField(blank=True, max_length=10)),
                ('id', django_audit_fields.fields.uuid_auto_field.UUIDAutoField(blank=True, db_index=True, editable=False, help_text='System auto field. UUID primary key.')),
                ('subject_identifier', models.CharField(max_length=50)),
                ('tracking_identifier', models.CharField(db_index=True, max_length=32)),
                ('action_identifier', models.CharField(db_index=True, max_length=50)),
                ('parent_action_identifier', models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True)),
                ('related_action_identifier', models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True)),
                ('action_item_reason', models.TextField(editable=False, null=True)),
                ('history_id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('report_datetime', models.DateTimeField(default=edc_utils.date.get_utcnow, verbose_name='Report Date and Time')),
                ('stop_date', models.DateField(verbose_name='Date decision to stop study medication')),
                ('last_dose_date', models.DateField(verbose_name='Date of last known dose')),
                ('reason', models.CharField(choices=[('pregnancy', 'Pregnancy'), ('sae', 'Participant is experiencing a serious adverse event'), ('clinician', 'Other condition that justifies discontinuation of treatment in the clinician’s opinion (specify below)'), ('investigator', ' Investigator decision'), ('patient', 'Patient decision'), ('OTHER', 'Other reason (specify below)')], max_length=25, verbose_name='Reason for stopping study medication')),
                ('permanently_discontinued', models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], max_length=15, verbose_name='Is study medication being permanently discontinued')),
                ('permanently_discontinued_reason', models.TextField(blank=True, null=True, verbose_name='Reason study medication being permanently discontinued')),
                ('expected_restart_date', models.DateField(blank=True, null=True, validators=[edc_model.validators.date.date_is_future], verbose_name='If not permanently discontinued, expected restart date')),
                ('history_date', models.DateTimeField()),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('action_item', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='edc_action_item.actionitem')),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('parent_action_item', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='edc_action_item.actionitem')),
                ('related_action_item', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='edc_action_item.actionitem')),
                ('site', models.ForeignKey(blank=True, db_constraint=False, editable=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='sites.site')),
            ],
            options={
                'verbose_name': 'historical Withdrawal of Study Drug',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



# Generated by Django 4.0.5 on 2022-07-02 15:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_subject', '0118_delivery_crf_status_delivery_crf_status_comments_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalstudymedication',
            name='roundup_divisible_by',
            field=models.IntegerField(default=32),
        ),
        migrations.AddField(
            model_name='studymedication',
            name='roundup_divisible_by',
            field=models.IntegerField(default=32),
        ),
    ]



# Generated by Django 3.2.13 on 2022-07-04 15:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_ae', '0009_auto_20220307_1929'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicalaefollowup',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical AE Follow-up Report', 'verbose_name_plural': 'historical AE Follow-up Reports'},
        ),
        migrations.AlterModelOptions(
            name='historicalaeinitial',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical AE Initial Report', 'verbose_name_plural': 'historical AE Initial Reports'},
        ),
        migrations.AlterModelOptions(
            name='historicalaesusar',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical AE SUSAR Report', 'verbose_name_plural': 'historical AE SUSAR Reports'},
        ),
        migrations.AlterModelOptions(
            name='historicalaetmg',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical AE TMG Report', 'verbose_name_plural': 'historical AE TMG Reports'},
        ),
        migrations.AlterModelOptions(
            name='historicaldeathreport',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Death Report', 'verbose_name_plural': 'historical Death Reports'},
        ),
        migrations.AlterModelOptions(
            name='historicaldeathreporttmg',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Death Report TMG (1st)', 'verbose_name_plural': 'historical Death Report TMG (1st)'},
        ),
        migrations.AlterModelOptions(
            name='historicaldeathreporttmgsecond',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Death Report TMG (2nd)', 'verbose_name_plural': 'historical Death Report TMG (2nd)'},
        ),
        migrations.AlterField(
            model_name='historicalaefollowup',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalaeinitial',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalaesusar',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalaetmg',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicaldeathreport',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicaldeathreporttmg',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicaldeathreporttmgsecond',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
    ]



# Generated by Django 3.2.13 on 2022-07-14 18:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_subject', '0122_auto_20220708_2144'),
    ]

    operations = [
        migrations.AddField(
            model_name='bloodresultsfbc',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='bloodresultsfbc',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='bloodresultsglu',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='bloodresultsglu',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='bloodresultshba1c',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='bloodresultshba1c',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='bloodresultsins',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='bloodresultsins',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='bloodresultslft',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='bloodresultslft',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='bloodresultslipid',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='bloodresultslipid',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='bloodresultsrft',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='bloodresultsrft',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultsfbc',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='historicalbloodresultsfbc',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultsglu',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='historicalbloodresultsglu',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultshba1c',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='historicalbloodresultshba1c',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultsins',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='historicalbloodresultsins',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultslft',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='historicalbloodresultslft',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultslipid',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='historicalbloodresultslipid',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultsrft',
            name='missing',
            field=models.TextField(editable=False, help_text='calculated string of field names that have been left blank', null=True),
        ),
        migrations.AddField(
            model_name='historicalbloodresultsrft',
            name='missing_count',
            field=models.IntegerField(default=0, editable=False, help_text='A count of fields left blank'),
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-27 14:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0045_historicalscreeningpartone_contact_number_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='hba1c_datetime',
            field=models.DateTimeField(blank=True, help_text='Date and time of result.', null=True, verbose_name='HbA1c date and time'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='hba1c_datetime',
            field=models.DateTimeField(blank=True, help_text='Date and time of result.', null=True, verbose_name='HbA1c date and time'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='hba1c_datetime',
            field=models.DateTimeField(blank=True, help_text='Date and time of result.', null=True, verbose_name='HbA1c date and time'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='hba1c_datetime',
            field=models.DateTimeField(blank=True, help_text='Date and time of result.', null=True, verbose_name='HbA1c date and time'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='hba1c_datetime',
            field=models.DateTimeField(blank=True, help_text='Date and time of result.', null=True, verbose_name='HbA1c date and time'),
        ),
    ]
